function output = WaTMRI_S2D(input,varargin)   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% minimize alpha*TV(y)+lambda*0.5*||G*Phi*y-z||_2^2+beta*(||z||_{2,1}+ ||Phi'*y||_1)
%%% + 0.5*||Ry-b||_2^2 
% input:
%          Phi: wavelet transform
%          Phi': inverse wavelet transform
%          R: partial Fourier transfrom
%          b: measurements
%          G:  matrix to transfer overlap groups to non-overlap
%             groups by extending x to z with G*x=z, where x,z are wavelet
%             coefficients        
%          groups: groups index of each element in z
%          beta,lambda,alpha: parameters

%%% Written by Chen Chen at University of Texas at Arlington
%%% Jan. 21, 2013,
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parsin.MAXITER=1; parsin.tv='iso';

l=input.l; u=input.u;
if((l==-Inf)&&(u==Inf))
    project=@(x)x;
elseif (isfinite(l)&&(u==Inf))
    project=@(x)(((l<x).*x)+(l*(x<=l)));
elseif (isfinite(u)&&(l==-Inf))
     project=@(x)(((x<u).*x)+((x>=u)*u));
elseif ((isfinite(u)&&isfinite(l))&&(l<u))
    project=@(x)(((l<abs(x))&(abs(x)<u)).*x)+((abs(x)>=u)*u*x/abs(x))+(l*x/abs(x)*(abs(x)<=l));
else
    error('lower and upper bound l,u should satisfy l<u');
end

t00 = cputime; 
n1=input.n1; n2=input.n2; N=n1*n2;
A=input.A; Phi=input.Phi;b=input.b; Atb=A'*b;

Lx=input.L; tnew=1;
alpha = input.alpha;
beta = input.beta; 
lambda = input.lamda;
G=input.G;GtG=input.GtG; 
groups=input.groups; Gmat=input.Gmat;
para.flag=1; 
% y=zeros(input.n1,input.n2);
y=Atb;
yr=y;
for itr = 1:input.maxitr  % total iter counter        

    told=tnew;
    yp=y;
    
    if itr==1
        z = zeros(size(G,1),1);
    else
       z=shrink_group(G*xg(:),beta/lambda,0,groups,Gmat);
    end

    tp = Phi'*reshape(G'*z,n1,n2);
    tp = Phi'*reshape(GtG*reshape(Phi*yr,N,1),n1,n2)-tp;
    tp = (lambda/Lx)*tp;
    
    yg=yr-(A'*(A*yr)-Atb)/Lx-tp;
    
     if (itr==1)
        [yt, P]=denoise_TV_One(yg, 2*alpha/Lx,-inf,inf,[],parsin); 
    else
        [yt, P]=denoise_TV_One(yg, 2*alpha/Lx,-inf,inf,P,parsin);
    end
    
    xg=Phi*yg;
%     x = shrink1D(xg,zeros(n1,n2), 2*beta/Lx);

%     y=((Phi'* x)+yt)/2.0;
    y = project(yg);

    tnew=(1+sqrt(1+4*told^2))/2; 
    yr=y+((told-1)/tnew)*(y-yp);     
 
    output.snr(itr)=snr(y, input.f);
    output.xtime(itr)=cputime-t00; 
    
     if itr>=input.no  
        break
     end
end

output.y=y; 

    function s1 = shrink1D(p,q,s)
        % shrink p toward q by up to s; p and q are vectors, s is scalar
        tmp = p - q; s1 = q + sign(tmp).*max(0,abs(tmp)-s);
    end
end